ExclusiveGhost

Struct ExclusiveGhost 

pub struct ExclusiveGhost<T>(/* private fields */);
ExclusiveGhost is a token that always provides exclusive access to a ghost value of type T. No two ExclusiveGhost tokens can have the same id.

Implementations

Source

impl<T> ExclusiveGhost<T>

pub closed spec fn id(self) -> Loc

Returns the unique identifier.

pub closed spec fn pcm(self) -> ExclR<T>

Returns the underlying ExclR<T> PCM element.

pub open spec fn value(self) -> T

{ self.pcm().value() }

Returns the ghost value of type T.

pub open spec fn view(self) -> T

{ self.value() }

Returns the ghost value of type T, an alias of Self::value().

pub open spec fn wf(self) -> bool

{ self.pcm() is Excl }

Type invariant: the PCM element must be Excl.

pub proof fn alloc(value: T) -> tracked res : Self

ensures
res.view() == value,
res.wf(),

Allocates a new ExclusiveGhost with the given value。

pub proof fn update(tracked &mut self, value: T) -> res : T

ensures
self.id() == old(self).id(),
self.view() == value,
self.wf(),

Updates the ghost value and returns the old value.

pub proof fn validate_with_other(tracked &mut self, tracked other: &Self)

ensures
*old(self) == *self,
self.id() != other.id(),
self.wf(),

The existence of two ExclusiveGhost tokens ensures that they do not have the same id.

pub proof fn validate(tracked &self)

ensures
self.wf(),

The token always satisfies Self::wf().